
*#endif

* Klasse zum sequentiellen Einlesen groer Dateien
* Da die Blockgre auf 4 KB begrenzt ist, kann man auch einfach die Zeilen ausschneiden.

#include "Fileio.ch"

function HBTextReader( cFile )
   local oTR := HBTxtReader():new( cFile )
return oTR

CLASS HBTxtReader
  PROTECTED:
     VAR nH
     VAR nLastError
     VAR IsEOF           // Buffer hat beim Einlesen EOF erreicht, Zeilen knnen noch da sein !
     VAR cRest
     VAR nBufferBytes    // Anzahl der gelesenen Byte im Buffer
     VAR cCRLF, nLenCRLF // Unix/Linux nur chr(10) = 1 Byte, Windows chr(13)+chr(10) = 2 Byte
     VAR Line
     METHOD ReadBuffer

  EXPORTED:
     METHOD Init
     METHOD Destroy
     METHOD GetLine
     METHOD GoTop
     METHOD FSize
     METHOD FError
     METHOD ErrMsg
     METHOD EOF
     METHOD IsCrLf
     METHOD IsUnix
     METHOD IsMac
     INLINE METHOD RecNo    ; RETURN ::line
     INLINE METHOD LenCrLf  ; RETURN ::nLenCRLF
     METHOD FileType

ENDCLASS

METHOD HBTxtReader:Init( cFileName )       // ffnet die Datei zum Lesen
   ::nLastError   := 0
   ::cRest        := ""
   ::nBufferBytes := 0
   ::Line         := 0
   ::nH    := fopen( cFileName , FO_READ + FO_SHARED )
   if ::nH = -1
      ::nLastError := FError()
      ::IsEOF      := .t.
   else
      ::IsEOF      := .f.
      ::ReadBuffer()
      do case
         case chr(13)+chr(10) $ ::cRest // Windows etc.
              ::cCRLF    := chr(13)+chr(10)
              ::nLenCRLF := 2
         case chr(10)         $ ::cRest // Unix
              ::cCRLF    := chr(10)
              ::nLenCRLF := 1
         case chr(13)         $ ::cRest // Mac
              ::cCRLF    := chr(13)
              ::nLenCRLF := 1
      end
   endif
RETURN SELF

METHOD HBTxtReader:Destroy()
   if ::nH <> -1
      FClose(::nH)
      ::nH := -1
   endif
   ::cRest := ""
RETURN SELF

METHOD HBTxtReader:ReadBuffer()
   local cBuffer, nBufferLen, nBytes
   if ::nH > -1
      nBufferLen := 4096
      cBuffer    := space(nBufferLen)
      nBytes     := FRead( ::nH, @cBuffer, nBufferLen)
      cBuffer    := StrTran(cBuffer,chr(26)," ")
      ::nBufferBytes += nBytes
      if nBufferLen = nBytes // mitten in Datei
         ::cRest += cBuffer
      else
         ::cRest += left(cBuffer,nBytes)
         ::IsEOF := .t.
         if FError() <> 0
            ::nLastError := FError()
         endif
      endif
      cBuffer := ""
   endif
Return

METHOD HBTxtReader:GetLine()
   local nPosCRLF
   local cLine := ""
   do while ! ::cCRLF $ ::cRest .and. ! ::IsEof  // Buffer einlesen, bis wir neue Zeilen haben oder die Datei gelesen wurde
      ::ReadBuffer()
   enddo
   nPosCRLF := at( ::cCRLF, ::cRest)
   if nPosCRLF > 0 // es gibt noch eine komplette Zeile, zur?ckgeben und k?rzen
      cLine   := left(::cRest,nPosCRLF-1)
      ::cRest := substr(::cRest,nPosCRLF+::nLenCRLF)
   else
      cLine   := ::cRest
      ::cRest := ""
   endif
   ::Line++
return cLine

METHOD HBTxtReader:GoTop()
   if ::nH <> -1
      FSeek(::nH, 0 , FS_SET )
   endif
   ::cRest   := ""  // zwingt zum neu einlesen
   ::Line    := 0
   ::ReadBuffer()
return NIL

METHOD HBTxtReader:FSize()
   local nSize := 0
   if ::nH <> -1
      nSize := FSize(::nH)
   endif
RETURN nSize

METHOD HBTxtReader:FError()
RETURN ::nLastError

METHOD HBTxtReader:EOF()
RETURN ::IsEOF .and. empty(::cRest)

METHOD HBTxtReader:IsCrLf()
RETURN (::cCRLF == chr(13)+chr(10))

METHOD HBTxtReader:IsUnix()
RETURN (::cCRLF == chr(10))

METHOD HBTxtReader:IsMac()
RETURN (::cCRLF == chr(13))

METHOD HBTxtReader:ErrMsg()
RETURN DosErrorMessage(::nLastError)

METHOD HBTxtReader:FileType()
   local cTxt := ""
   do case
      case ::IsCrLf()
           cTxt := "CRLF"
      case ::IsUnix()
           cTxt := "UNIX/Linux"
      case ::IsMac()
           cTxt := "MAC"
   end
return cTxt
*********************************************************************
******** Display a warning message
********        ctitle:

*message := {}
*AADD(message,'1- ')
*AADD(message,'2- ')
*AADD(message,'3- ')
*LB_Warning( message )

FUNCTION LB_Warning( message, ctitle )

  LOCAL aMsg := {}
  DEFAULT cTitle TO ''
  IF valtype(message) # 'A'
    aadd(aMsg,message)
  ELSE
    aMsg := message
  ENDIF
  IF LEN(ALLTRIM(cTitle)) > 0
     DC_MsgBox( ,,aMsg,cTitle)
  ELSE
     DC_MsgBox( ,,aMsg,'(C)     "-++"')
  ENDIF

RETURN NIL

***********************************************************************************************************
******  - (   XSample_14() xdemo.exe)
***********************************************************************************************************
STATIC FUNCTION Time_Progress (Time_Progress, Wsego, oProgress, lOk )

    LOCAL nMaxCount := Wsego

    xtime = Time_Progress

    **     ,        0.1  (  PercTimeVisio())

    T2tp = (DOY(DATE())-1)*86400+SECONDS()          //  
    mNumPP = IF(mNumPP+1<=Wsego, ++mNumPP, mNumPP)
    IF T2tp - T1tp > 0.1 .OR. mNumPP = N_ALL        //     100%

*      aSay[mPTVnumb]:SetCaption(mPTVmess+' '+ALLTRIM(STR(mNumPP/Wsego*100,15,7))+'%')

       ***   
       
       *****     ,   
       *****     ,    
*      T_Mess1 = ":"+" "+TIME()           // 
       
       *****     
       PUBLIC T_Mess2 := "ch:mi:se"
       Sec_2   = (DOY(DATE())-1)*86400+SECONDS() - Sec_1
       ch2 = INT(Sec_2/3600)                    // 
       mm2 = INT(Sec_2/60)-ch2*60               // 
       cc2 = Sec_2-ch2*3600-mm2*60              // 
       T_Mess2 = ":"+" "+ALLTRIM(STRTRAN(T_Mess2,"ch",STR(ch2,19)))
       T_Mess2 = STRTRAN(T_Mess2,"mi",STRTRAN(STR(mm2,2)," ","0"))
       T_Mess2 = STRTRAN(T_Mess2,"se",STRTRAN(STR(cc2,2)," ","0"))
       *@19,2 SAY T_Mess2+" : "+ALLTRIM(STR(Sec_2,17))+" ."
       
       PUBLIC T_Mess3 := "ch:mi:se"             // 
       Sec_3 = Sec_2*Wsego/xtime                // ...  
       ch3 = INT(Sec_3/3600)                    // 
       mm3 = INT(Sec_3/60)-ch3*60               // 
       cc3 = Sec_3-ch3*3600-mm3*60              // 
       T_Mess3 = ALLTRIM(STRTRAN(T_Mess3,"ch",STR(ch3,19)))
       T_Mess3 = STRTRAN(T_Mess3,"mi",STRTRAN(STR(mm3,2)," ","0"))
       T_Mess3 = STRTRAN(T_Mess3,"se",STRTRAN(STR(cc3,2)," ","0"))
       *@20,2 SAY T_Mess3+" : "+ALLTRIM(STR(Sec_3,17))+" ."
       
       PUBLIC T_Mess4 := "ch:mi:se"             // 
       Sec_4 = Sec_1 + Sec_3 - (DOY(DATE())-1)*86400
       ch4 = INT(Sec_4/3600)                    // 
       mm4 = INT(Sec_4/60)-ch4*60               // 
       cc4 = Sec_4-ch4*3600-mm4*60              // 
       T_Mess4 = ":"+" "+ALLTRIM(STRTRAN(T_Mess4,"ch",STR(ch4,19)))
       T_Mess4 = STRTRAN(T_Mess4,"mi",STRTRAN(STR(mm4,2)," ","0"))
       T_Mess4 = STRTRAN(T_Mess4,"se",STRTRAN(STR(cc4,2)," ","0"))
       *@21,2 SAY T_Mess4+" : "+ALLTRIM(STR(Sec_4,17L())+" . .")
       
       PUBLIC T_Mess5 := ".  1- : ch:mi:se"
       Sec_5 = Sec_2/xtime
       ch5 = INT(Sec_5/3600)                    // 
       mm5 = INT(Sec_5/60)-ch5*60               // 
       cc5 = Sec_5-ch5*3600-mm5*60              // 
       T_Mess5 = ALLTRIM(STRTRAN(T_Mess5,"ch",STR(ch5,19)))
       T_Mess5 = STRTRAN(T_Mess5,"mi",STRTRAN(STR(mm5,2)," ","0"))
       T_Mess5 = STRTRAN(T_Mess5,"se",STRTRAN(STR(cc5,2)," ","0"))
       *@22,2 SAY T_Mess5+" : "+ALLTRIM(STR(Sec_5,17))+" ."
       
       PUBLIC T_Mess6 := "ch:mi:se"             // 
       Sec_6 = Sec_3 - Sec_2
       ch6 = INT(Sec_6/3600)                    // 
       mm6 = INT(Sec_6/60)-ch6*60               // 
       cc6 = Sec_6-ch6*3600-mm6*60              // 
       T_Mess6 = ":"+" "+ALLTRIM(STRTRAN(T_Mess6,"ch",STR(ch6,19)))
       T_Mess6 = STRTRAN(T_Mess6,"mi",STRTRAN(STR(mm6,2)," ","0"))
       T_Mess6 = STRTRAN(T_Mess6,"se",STRTRAN(STR(cc6,2)," ","0"))
       *@23,2 SAY T_Mess6+" : "+ALLTRIM(STR(Sec_6,17))+" ."
       
       Mess98 = T_Mess1+SPACE(142-LEN(T_Mess1)-LEN(T_Mess4))+T_Mess4  // ,  () 145
       oSay98:SetCaption(Mess98);oSay98:SetCaption(oSay98:caption)
       
       Mess99 = T_Mess2+SPACE(144-LEN(T_Mess2)-LEN(T_Mess6))+T_Mess6  // ,  ()  146
       oSay99:SetCaption(Mess99);oSay99:SetCaption(oSay99:caption)
       
       DC_GetProgress( oProgress, Time_Progress, Wsego )              //   Progress-bar
       
*      Sec_1   // 
*      Sec_4   // 
*      Sec_2   //     
*      Sec_6   //    
       
*      mTimeProgress=ROUND(Sec_2/(Sec_2+Sec_6)*100,0)
*      DC_GetProgress( oProgress, mTimeProgress, 100 )                //   Progress-bar
       
*      DC_GetProgress( oProgress, ROUND(Sec_2,0), ROUND(Sec_2+Sec_6,0) )                //   Progress-bar
       
*      MsgBox(STR(ROUND(Sec_2,0))+STR(ROUND(Sec_2+Sec_6,0)))
*      MsgBox(STR(mTimeProgress))
*      MILLISEC(1000)
       
*      K=100/Wsego;mTimeProgress=ROUND(K*Time_Progress,0)
*      DC_GetProgress( oProgress, mTimeProgress, 100 )                //   Progress-bar
       
       DC_AppEvent( @lOk, 0, .01 )

       T1tp = T2tp
    ENDIF

RETURN lOk

******************************************************************************************
********    mMess aSay[mPTVnumb]   , 
********     ,      
********    0,1    100% 
******************************************************************************************
FUNCTION PercTimeVisio(mPTVnumb, mPTVmess, mPTVnALL)
******************************************************************************************

*****    

*CLOSE ALL
*USE Obi_Zag EXCLUSIVE NEW
*SELECT Obi_Zag

*N_Obj = RECCOUNT()                           // 1, N_Obj ################################
*mNumPP = 0
*N_ALL = N_Obj
**       1
*mMess = L('5/8:   "ABS".  :')
*PUBLIC T1 := (DOY(DATE())-1)*86400+SECONDS()         //     
*PUBLIC T2 := (DOY(DATE())-1)*86400+SECONDS()+1       //   (1-     T1   )

*DBGOTOP()
*DO WHILE .NOT. EOF()                         //      

    T2 = (DOY(DATE())-1)*86400+SECONDS()      //  
    mNumPP = IF(mNumPP+1<=mPTVnALL, ++mNumPP, mNumPP)
    IF T2 - T1 > 0.1 .OR. mNumPP = N_ALL      //     100%
       aSay[mPTVnumb]:SetCaption(mPTVmess+' '+ALLTRIM(STR(mNumPP/mPTVnALL*100,15,7))+'%')
       T1 = T2
    ENDIF

*   DBSKIP(1)
*ENDDO

RETURN NIL
******************************************************************************************
Function CLoseAll()            //        
******************************************************************************************
LOCAL bError , aWorkareas , x, y

bError := ErrorBlock( {|e| Break(e)} )
aWorkAreas := workspacelist()
y := len(aWorkAreas)

for x = 1 to y
  BEGIN SEQUENCE
    (aWorkAreas[x])->(DbClearRelation())
    (aWorkAreas[x])->(DbCloseArea())
  ENDSEQUENCE
next
ErrorBlock(bError)
bError   := nil
Return nil
************************************************************************************
*****************************************************************************************************
********     ,  -     
*****************************************************************************************************
FUNCTION Running(lValue)
STATIC lRunning := .F.
   IF PCOUNT() > 0
      lRunning := lValue
   ENDIF
RETURN lRunning

